package Object;
use vars qw( $id );
$id = 0;
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{ID} = $id++;
  bless $self, $class;
  return $self;
}
sub print {
  my $self = shift;
  print "%% Object->new();\n";
}

######################################################################

package Text;
use vars qw( @ISA );
@ISA = qw(Object);

sub new {
  my $proto = shift;
  my $self = $proto->SUPER::new;
  $self->{X} = shift;
  $self->{Y} = shift;
  $self->{TEXT} = shift;
  $self->{ANCHOR} = shift;
  $self->draw;
  Stack::add($self);
  return $self;
}

sub draw {
  my $self = shift;
  main::draw_text( $self->{ID},
                   $self->{X}, $self->{Y}, $self->{TEXT}, $self->{ANCHOR});
  return $self;
}

sub print {
  my $self = shift;
  printf "%% Text->new(%s,%s,\"%s\",\"%s\");\n",
         $self->{X}, $self->{Y}, main::stringify($self->{TEXT}), $self->{ANCHOR};
}

sub latex {
  my $self = shift;
  print "($self->{X},$self->{Y}) ";
  if( $self->{ANCHOR} eq "e" ){
    print "*+!R{$self->{TEXT}\\ }!C";
  } elsif( $self->{ANCHOR} eq "w" ){
    print "*+!L{\\ $self->{TEXT}}!C";
  } else {
    print "*+{$self->{TEXT}}";
  }
  print '="'. $self->{ID} .'",'. "\n";
}


######################################################################

package Arrow;
use vars qw( @ISA );
@ISA = qw(Object);

sub new {
  my $proto = shift;
  my $self = $proto->SUPER::new;
  $self->{X1} = shift;
  $self->{Y1} = shift;
  $self->{X2} = shift;
  $self->{Y2} = shift;
  $self->draw;
  Stack::add($self);
  return $self;
}

sub draw {
  my $self = shift;
  main::draw_arrow( $self->{ID},
                    $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2});
  return $self;
}

sub print {
  my $self = shift;
  printf "%% Arrow->new(%s,%s,%s,%s);\n",
         $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2};
}

sub latex {
  my $self = shift;
  my $id;
  my $s = "($self->{X1},$self->{Y1})";
  my $t = "($self->{X2},$self->{Y2})";
  foreach $id (keys %Stack::stack){
    my $obj = $Stack::stack{$id};
    if( defined $obj->{X} ){
      if( $obj->{X} == $self->{X1} and $obj->{Y} == $self->{Y1} ){
        if( $obj->{ANCHOR} eq 'w' or ! ref($s) ){ $s = $obj }
      } elsif( $obj->{X} == $self->{X2} and $obj->{Y} == $self->{Y2} ){
        if( $obj->{ANCHOR} eq 'e' or ! ref($t) ){ $t = $obj }
      }
    }
  }
  $s = '"'. $s->{ID} .'"' if ref($s);
  $t = '"'. $t->{ID} .'"' if ref($t);
  print "\\ar $s;$t\n";
}

######################################################################

package Line;
use vars qw( @ISA );
@ISA = qw(Object);

sub new {
  my $proto = shift;
  my $self = $proto->SUPER::new;
  $self->{X1} = shift;
  $self->{Y1} = shift;
  $self->{X2} = shift;
  $self->{Y2} = shift;
  $self->draw;
  Stack::add($self);
  return $self;
}

sub draw {
  my $self = shift;
  main::draw_line( $self->{ID},
                   $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2});
  return $self;
}

sub print {
  my $self = shift;
  printf "%% Line->new(%s,%s,%s,%s);\n",
         $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2};
}

sub latex {
  my $self = shift;
  print "\\ar\@{-}($self->{X1},$self->{Y1});($self->{X2},$self->{Y2})\n";
}

######################################################################

package DoubleLine;
use vars qw( @ISA );
@ISA = qw(Object);

sub new {
  my $proto = shift;
  my $self = $proto->SUPER::new;
  $self->{X1} = shift;
  $self->{Y1} = shift;
  $self->{X2} = shift;
  $self->{Y2} = shift;
  $self->draw;
  Stack::add($self);
  return $self;
}

sub draw {
  my $self = shift;
  main::draw_double_line( $self->{ID},
                          $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2});
  return $self;
}

sub print {
  my $self = shift;
  printf "%% DoubleLine->new(%s,%s,%s,%s);\n",
         $self->{X1}, $self->{Y1}, $self->{X2}, $self->{Y2};
}

sub latex {
  my $self = shift;
  print "\\ar\@{=}($self->{X1},$self->{Y1});($self->{X2},$self->{Y2})\n";
}

######################################################################

package main;